////////////////////////////////////////////////////////////
// Exemple de programme pour la radiocommande d'un mobile //
// Programmation de l'arduino récepteur                   //
// Auteur :Jean Garrrigues                                //
////////////////////////////////////////////////////////////

// Ce programme est libre de tous droits.
// Vous pouvez l'utiliser et le modifier librement.

// DIRECTIVES POUR LE RECEPTEUR RADIO
#include <VirtualWire.h> // bibliothèque de communication sans fil

/* STRUTURE DES MESSAGES : les messages radio à émettre contiennent tous 7 octets :
  ID_RESEAU , ID_EMETTEUR , ID_RECEPTEUR , vitesseGauche , vitesseDroite , bouton , octet supplémentaire
  (l'octet supplémentaire est prévu pour une utilisation future d'un joystick 3 axes).
*/

// Câblage du circuit récepteur
#define BROCHE_DONNEES_RECEPTEUR 11  // (ou toute autre broche disponible)
// Vcc récepteur ---- arduino 5V
// GND récepteur ---- arduino GND
// DATA récepteur ---- arduino 11

// Définition des noms symboliques des identificateurs
// ATTENTION : les valeurs qui suivent doivent être les mêmes sur l'émetteur et le récepteur.
// Vous devez choisir d'autres valeurs pour ne pas interférer avec des voisins qui utiliseraient ce programme sans le modifier !
#define ID_RESEAU 72     // choisissez un entier entre 0 et 255
#define ID_EMETTEUR 58   // choisissez un entier entre 0 et 255
#define ID_RECEPTEUR 214 // choisissez un entier entre 0 et 255

// Autres noms symboliques
#define VITESSE_DE_TRANSMISSION 2000 // en bits/seccpnde (doit être la même sur tous les émetteurs/récepteurs du réseau)
#define DELAI_RUPTURE_DE_LIAISON 1000 // en millisecondes

// Déclaration du tableau d'octets pour stocker le message reçu
uint8_t message[VW_MAX_MESSAGE_LEN];
uint8_t tailleMessage = VW_MAX_MESSAGE_LEN;
/*
   NOTE Le tableau est surdimentionné à VW_MAX_MESSAGE_LEN octets afin d'éviter un dépassement de tableau par
   la réception d'un message parasite
*/


// Câblage de la LED d'alarme de rupture de laison radio
#define BROCHE_LED_ALARME 9 // ou toute autre broche diponible (digitale ou analogique)
// LED + ---- Résistance>500 Ohms ---- arduino 9 (valeur de BROCHE_LED_ALARME)
// LED GND ---- arduino GND


// INSERER ICI les directives et les déclarations nécessaires pour la carte de pilotage des moteurs
/*
   Certaines cartes de pilotage de moteurs (notamment les shields) nécessitent généralement l'inclusion d'une bibliothèque.
   D'autres ne nécessitent pas de biblothèque mais il faut au moins déclarer sur quelles broches elles sont raccordées à l'arduino.

   Certaines cartes intelligentes récentes communiquent avec l'arduino par l'intermédiaire d'un protocole I2C ou SPI ou autre.
   Dans ce cas les commandes pour piloter les moteurs sont spécifiques et sont envoyées sur le canal de communinacation. Consulter
   la documentation de la carte.

   Quelle que soit la carte utilisée (shield ou non), le réglage de la puissance d'un moteur se fait en l'alimentant en
   PWM (Pulse With Modulation) : la tension d'alimentation n'est fournie qu'une partie du temps (sur une période de 2 ms environ).
   Le pourcentage du temps d'alimentation est réglé par un nombre entre 0 (0%) et 255 (100%).
   (les sorties PWM sont les broches digitales marquées d'un ~). On les pilote avec l'instruction analogWrite.
*/

// Les contantes symboliques qui suivent sont à ajuster :
// elles dépendent des moteurs utilisés et du couple résistant de votre modèle.
#define VIT_MIN_GAUCHE 70    // valeur minimale pour que le moteur tourne. A ajuster par des tests avec le modèle
#define VIT_MAX_GAUCHE 255   // valeur maximale pour le moteur. A ajuster par des tests avec le modèle
#define VIT_MIN_DROITE 70    // valeur minimale pour que le moteur tourne. A ajuster par des tests avec le modèle
#define VIT_MAX_DROITE 255   // valeur maximale que le moteur tourne. A ajuster par des tests sur le modèle
// le but des valeurs minimales est d'éviter d'alimenter le moteur trop faiblement sans qu'il tourne.
// le but des valeurs maximales est de régler la vtesse maxi de votre modèle.
#define MARGE 5 // tolérance autour des valeurs centrales du joystick (vous pouvez la modifier)


// Variables globales : déclaration et initialisation des vitesses des roues.
int vitGauche = 0, vitDroite = 0;
boolean roueGaucheEnMarcheAvant = true;
boolean roueDroiteEnMarcheAvant = true;


#define DEBUG_VIT false// remplacer false par true pour déclencher les écritures dans la fenêtre "Moniteur Série"
/*
    Note : Dans la suite, les instructions qui sont comprises entre #if DEBUG et #endif ne seront prises en compte et
    compilées que si le symbole ÒDEBUG est défini comme true.
    Pensez à remettre #define DEBUG false avant le dernier téléversement quand votre programme est au point.
*/

///////////////////////////////////////////////////////////////

void setup() {

#if DEBUG
  Serial.begin(9600);
#endif
  /*
    Rappel : les instructions comprises entre #if DEBUG et #endif ne seront prises en compte et compilées que si
    DEBUG signifie true.
  */

  // initialisation du circuit récepteur
  pinMode(BROCHE_DONNEES_RECEPTEUR, INPUT);
  vw_setup(VITESSE_DE_TRANSMISSION);
  vw_set_rx_pin(BROCHE_DONNEES_RECEPTEUR);
  vw_rx_start(); // demarrage de l'écoute

  pinMode(BROCHE_LED_ALARME, OUTPUT);
  digitalWrite(BROCHE_LED_ALARME, LOW);

  // AJOUTER ICI les déclarations de mode et les initialisations nécessaires pour la carte moteur utilisée

}

///////////////////////////////////////////////////////////////

void loop() {

  // On attend de recevoir un message
  if ( ! vw_wait_rx_max(DELAI_RUPTURE_DE_LIAISON)) {
    // la fonction vw_wait_rx_max est bloquante : on n'en sort que si on a reçu un  message ou
    // si DELAI_RUPTURE_DE_LIAISON est dépassé. Si le délai est atteint elle répond false.
    arretDUrgence();  // le mobile s'arrête s'il perd la liaison radio pendant plus de DELAI_RUPTURE_DE_LIAISON millisecondes
  }

  if (vw_get_message(message, &tailleMessage)) { // Si on a reçu un message
    // On récupère le message si le message reçu est correct (checksum vérifié)
    // Le message est copié dans le tableau d'octets message, qu'il soit corrompu ou non
    // Le nombre d'octets reçus est assigné à la variable tailleMessage
    // Penser à la remttre à la valeur maximale avant toute nouvelle réception (tailleMessage = VW_MAX_MESSAGE_LEN;)
    // si la taille des messages est variable. Ici ce n'est pas utile.
    //vw_rx_stop();  //On arrête l'écoute
    digitalWrite(BROCHE_LED_ALARME, LOW);// On éteint la LED d'alarme
    interpreteMessage();
  }
  else {
    // le message est mal recu (checksum incorrect) ou bien il s'agit d'un parasite
    // on l'ignore sans rien faire
  }
  tailleMessage = VW_MAX_MESSAGE_LEN;

  /*
     La boucle loop ne fait donc qu'attendre un message. On n'en sort que par arretDUrgence() ou par interpreteMessage();
  */
}

/////////////////////////////////////////////////////

void arretDUrgence(void) {

  digitalWrite(BROCHE_LED_ALARME, HIGH); // allumage de la LED d'alarme

  // INSERER ICI les instructions pour arrêter les moteurs et tout autre dispositif devant
  // être éteint pendant une rupture de liaison radio

}

/////////////////////////////////////////////////////

void interpreteMessage(void) {

  // vérifications préliminaires
  if (message[0] != ID_RESEAU ||
      message[1] != ID_EMETTEUR ||
      message[2] != ID_RECEPTEUR ||
      tailleMessage != 7
     ) {
    //  Ce message ne nous est pas destiné : il vient peut-être
    //  d'un autre utilisateur de virtualWire, (le cheksum est correct)
    //  d'un autre réseau de communication,
    //  d'un autre émetteur
    //  destiné à un autre destinataire dans le réseau
    return ; // On ne fait rien, retour à la boucle loop
  }

  // Récupération des vitesses :
  vitGauche = message[3] - 127;
  vitDroite = message[4] - 127;

#if DEBUG
  // les vitesses gauche et droite doivent être des entiers compris entre -127 et 127
  Serial.print("vitGauche = ");
  Serial.print(vitGauche);
  Serial.print("   vitDroite = ");
  Serial.println(vitDroite);
#endif

  // Traitement de la vitesse de la roue gauche
  /*
     Dans les instructions qui suivent, les commandes pour faire tourner les moteurs gauche et
     droite à la bonne vitesse et dans le bon sens ne sont pas précisées car elles
     dépendent de la carte de pilotage des moteurs que vous utilisez.
     Vous devez les compléter !
  */
  if (vitGauche <= -MARGE) { // le moteur gauche doit tourner en marche arrière
#if DEBUG
    roueGaucheEnMarcheAvant = false;
#endif
    vitGauche = map(vitGauche , -MARGE , VIT_MIN_GAUCHE , VIT_MAX_GAUCHE , VIT_MIN_GAUCHE);

    // INSERER ICI les instructions pour faire tourner la roue gauche en arrière à la vitesse vitGauche
  }
  else if (vitGauche >= MARGE) { //moteur gauche doit tourner en marche avant
#if DEBUG
    roueGaucheEnMarcheAvant = true;
#endif
    vitGauche = map(vitGauche , MARGE, 127 , VIT_MIN_GAUCHE , VIT_MAX_GAUCHE);

    // INSERER ICI les instructions pour faire tourner la roue gauche en avant à la vitesse vitGauche
  }
  else {
    vitGauche = 0;

    // INSERER ICI les instructions pour arrêter la roue gauche
  }

  // Traitement de la vitesse de la roue droite
  if (vitDroite <= -MARGE) { // le moteur droit doit tourner en marche arrière
#if DEBUG
    roueDroiteEnMarcheAvant = false;
#endif
    vitGauche = map(vitGauche , -127 , -MARGE , VIT_MAX_DROITE , VIT_MIN_DROITE);

    // INSERER ICI les instructions pour faire tourner la roue droite en arrière à la vitesse vitDroite
  }
  else if (vitDroite >= MARGE) { // le moteur droit doit tourner en marche avant
#if DEBUG
    roueDroiteEnMarcheAvant = true;
#endif
    vitDroite = map(vitDroite , MARGE, 127 , VIT_MIN_DROITE , VIT_MAX_DROITE);

    // INSERER ICI les instructions pour faire tourner la roue droite en avant à la vitesse vitDroite
  }
  else {
    vitDroite = 0;

    // INSERER ICI les instructions pour arrêter la roue droite
  }

#if DEBUG
  // les vitesses gauche et droite doivent maintenant être des entiers compris entre les  VIT_MIN et VIT_MAX respectifs
  Serial.print("vitGauche = ");
  Serial.print(vitGauche);
  if (roueGaucheEnMarcheAvant) {
    Serial.print(" AV");
  }
  else {
    Serial.print(" AR");
  }
  Serial.print("     vitDroite = ");
  Serial.print(vitDroite);
  if (roueDroiteEnMarcheAvant) {
    Serial.print(" AV");
  }
  else {
    Serial.println(" AR");
  }
#endif

  // Exploitation de l'octet bouton (message[5])
  if (bitRead(message[5], 0)) { // Le bouton du joystick est appuyé

    // INSERER ICI les actions à faire quand le bouton est pressé
  }

 /*
  * Note : Ici on n'a utilisé que le bit 0 (celui de droite) de l'octet bouton (message[5]).
  * Si le boitier de radiocommande comporte d'autres boutons, on peut utiliser les autres bits pour faire d'autres actions.
  * Exemple pour lire le bit 1 Ò:
  * if (bitRead(message[5], 1)){ ... }
  * 
  */

  /*
   * Note : dans ce programme, le 7ème octet du message (message[6]) est inutilisé. Il est initialement prévu pour l'utilisation 
   * éventuelle d'un joystik plus complet (rotation du manche), qui pourrait commander la rotation de la tourelle d'un 
   * char militaire ou la rotation de la flèche d'une grue.
   */

}
